<?php

namespace App\Jobs;

use App\Feedback;
use App\Mark;
use App\Wowa;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use RUCD\Training\SolutionDistance;
use RUCD\Training\Trainer;
use RUCD\Training\TrainerParameters;

class WowaJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    protected $feedback_number;
    protected $wowa;
    protected $is_true_alert = [];
    protected $evidences = [];
    protected $bool_auc;
    protected $bool_pr;
    protected $chunk_size = 100;

    /**
     * Create a new job instance.
     *
     * @param Wowa $wowa
     * @param bool $auc
     * @param bool $pr
     */
    public function __construct(Wowa $wowa)
    {
        $this->wowa = $wowa;
        $this->feedback_number = Feedback::count();
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $logg = $this->wowa->logger();
        $this->wowa->status = Wowa::STATE_RUNNING;
        $this->wowa->start_time = time();
        $this->wowa->save();
        if ($this->feedback_number == 0) {
            $this->wowa->status = Wowa::STATE_FAILED;
            $this->wowa->end_time = time();
            $this->wowa->save();
            return;
        }

        try {
            $this->wowa->status = Wowa::STATE_COLLECTION;
            $this->wowa->save();
            Feedback::chunk($this->chunk_size, function ($feedbacks) use ($logg) {
                foreach ($feedbacks as $feed) {
                    $evidence_references = $feed->report()->references;
                    $this->is_true_alert[] = $feed->is_true_alert;
                    $scores_references = [];
                    foreach ($evidence_references as $evidence_reference) {
                        $scores_references[] = Mark::get()->findEvidenceById($evidence_reference)->score;
                    }
                    $this->evidences[] = $scores_references;
                }
                $logg->info(($feed->id * 100) / $this->feedback_number . " | Evidences collection");
            });
        } catch (\Exception $e) {
            $this->wowa->status = Wowa::STATE_FAILED;
            $logg->error("Error during retrieving evidences");
            $this->wowa->save();
        }
        //Training Parameters
        $training_prameters = new TrainerParameters(
            $logg,
            $this->wowa->population,
            $this->wowa->crossover_rate,
            $this->wowa->mutation_rate,
            TrainerParameters::SELECTION_METHOD_RWS,
            $this->wowa->generation_number,
            TrainerParameters::INITIAL_POPULATION_GENERATION_RANDOM
        );
        $trainer = new Trainer($training_prameters, new SolutionDistance(count($this->evidences[0])));
        $this->wowa->status = Wowa::STATE_RUNNING;
        $this->wowa->save();
        try {
            $solution = $trainer->run($this->evidences, $this->is_true_alert);
        } catch (\Exception $e) {
            $this->wowa->status = Wowa::STATE_FAILED;
            $logg->error("Error during wowa training");
            $this->wowa->save();
        }
        $this->wowa->w_weights = $solution->weights_w;
        $this->wowa->p_weights = $solution->weights_p;
        $this->wowa->score = $solution->getDistance();
        $this->wowa->end_time = time();
        $solution->computeAUC($this->evidences, $this->is_true_alert);
        $solution->computePRAUC($this->evidences, $this->is_true_alert);
        $this->wowa->roc = $solution->getAucRoc();
        $this->wowa->pr = $solution->getAucPr();
        $this->wowa->status = Wowa::STATE_SUCCESS;
        $this->wowa->save();
    }

    //Method to set Job status to Failed in case of job failure
    public function failed()
    {
        $this->wowa->status = Wowa::STATE_FAILED;
        $this->wowa->save();
    }
}
